@@ -39,38 +39,35 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
  * SECTION:element-tiger
+ * @title: tiger
  * @see_also: katedec
  *
- * <refsect2>
- * <para>
- * This element decodes and renders Kate streams
- * <ulink url="http://libkate.googlecode.com/">Kate</ulink> is a free codec
- * for text based data, such as subtitles. Any number of kate streams can be
- * embedded in an Ogg stream.
- * </para>
- * <para>
- * libkate (see above url) and <ulink url="http://libtiger.googlecode.com/">libtiger</ulink>
+ * This element decodes and renders Kate streams.
+ * [Kate](http://libkate.googlecode.com/) is a free codec for text based data,
+ * such as subtitles. Any number of kate streams can be embedded in an Ogg
+ * stream.
+ *
+ * libkate (see above url) and [libtiger](http://libtiger.googlecode.com/)
  * are needed to build this element.
- * </para>
- * <title>Example pipeline</title>
- * <para>
+ *
+ * ## Example pipeline
+ *
  * This pipeline renders a Kate stream on top of a Theora video multiplexed
  * in the same stream:
- * <programlisting>
- * gst-launch \
+ * |[
+ * gst-launch-1.0 \
  *   filesrc location=video.ogg ! oggdemux name=demux \
  *   demux. ! queue ! theoradec ! videoconvert ! tiger name=tiger \
  *   demux. ! queue ! kateparse ! tiger. \
  *   tiger. ! videoconvert ! autovideosink
- * </programlisting>
- * </para>
- * </refsect2>
+ * ]|
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -91,14 +88,14 @@ GST_DEBUG_CATEGORY_EXTERN (gst_katetiger_debug);
 
 #define GST_KATE_TIGER_MUTEX_LOCK(element) \
   do { \
-    /*GST_LOG_OBJECT ((element), "locking from %s:%d\n",__FILE__,__LINE__);*/ \
+    /*GST_LOG_OBJECT ((element), "locking from %s:%d",__FILE__,__LINE__);*/ \
     g_mutex_lock ((element)->mutex); \
-    /*GST_LOG_OBJECT ((element), "ready from %s:%d\n",__FILE__,__LINE__);*/ \
+    /*GST_LOG_OBJECT ((element), "ready from %s:%d",__FILE__,__LINE__);*/ \
   } while(0)
 
 #define GST_KATE_TIGER_MUTEX_UNLOCK(element) \
   do { \
-    /*GST_LOG_OBJECT ((element), "unlocking from %s:%d\n",__FILE__,__LINE__);*/ \
+    /*GST_LOG_OBJECT ((element), "unlocking from %s:%d",__FILE__,__LINE__);*/ \
     g_mutex_unlock ((element)->mutex); \
   } while(0)
 
@@ -219,12 +216,10 @@ gst_kate_tiger_base_init (gpointer gclass)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&kate_sink_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&video_sink_factory));
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class, &kate_sink_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &video_sink_factory);
   gst_element_class_set_static_metadata (element_class, "Kate stream renderer",
       "Mixer/Video/Overlay/Subtitle",
       "Decodes and renders Kate streams on top of a video",
