@@ -20,17 +20,17 @@
  */
 /**
  * SECTION:element-spacescope
+ * @title: spacescope
  * @see_also: goom
  *
  * Spacescope is a simple audio visualisation element. It maps the left and
  * right channel to x and y coordinates.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 audiotestsrc ! audioconvert ! spacescope ! ximagesink
  * ]|
- * </refsect2>
+ *
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -148,6 +148,8 @@ gst_space_scope_class_init (GstSpaceScopeClass * g_class)
           "Drawing styles for the space scope display.",
           GST_TYPE_SPACE_SCOPE_STYLE, STYLE_DOTS,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gst_type_mark_as_plugin_api (GST_TYPE_SPACE_SCOPE_STYLE, 0);
 }
 
 static void
